@@ -1,64 +1,116 @@
 import matplotlib.pyplot as plt
 import math
-import difflib
-from plotex.configuration import BackendConfiguration
+import difflib, os
+import json
+import requests
 
+from plotex.configuration import BackendConfiguration
+from plotex.utils.general import save_file
 
 class Sizing():
     
+    CONFIG_URL = 'https://gist.githubusercontent.com/rg089/92540eef5ee88de5d2770a453c85c489/raw/b127ba7b0e7eff3c5eddf1202f01595e5c60c949/size_config.json'
+    CONFIG_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), "config_files/size_config.json")
+    
+    
     def __init__(self, config=None, **config_kwargs):
-        """
-        initialize the sizing class
+        """initialize the sizing class
 
-        :param config: the config file, defaults to None
+        Args:
+            config: the config file, defaults to None
         """
         self.config = config
         if self.config is None:
             self.config = BackendConfiguration(**config_kwargs)
             self.config.initialize()
         self.params = None
-    
-    
+        self.config = self.__load_config()
+        
+        
+    def __load_config(self):
+        if os.path.exists(Sizing.CONFIG_PATH):
+            with open(Sizing.CONFIG_PATH, 'r') as f:
+                config = json.load(f)
+        else:
+            r = requests.get(Sizing.CONFIG_URL)
+            config = r.json()
+            save_file(content=config, fpath=Sizing.CONFIG_PATH)
+            
+        return config
+            
+            
     def __save_params(self):
-        """
-        save a copy of the current rcParams
-        """
+        """save a copy of the current rcParams"""
         self.params = plt.rcParams.copy()
         
         
     def __load_params(self):
-        """
-        load the current saved copy of the rcParams
-        """
+        """load the current saved copy of the rcParams"""
         if self.params is not None:
             plt.rcParams.update(self.params)
     
     
-    def __get_width_publisher(self, publisher):
-        """
-        get the width in pts given the name of the publisher
+    def __get_width_publisher(self, publisher, width=None):
+        """get the width in pts given the name of the publisher
 
-        :param publisher: the name of the publisher
-        :return: the width in pts
+        Args:
+            publisher (str): the name of the publisher
+            width (float): if width is specified, then it is cached
+
+        Returns:
+            float: the width in pts
         """
-        if publisher == 'thesis':
-            width = 426.79135
-        elif publisher == 'acl':
-            width = 455.244
+        publisher = publisher.lower()
+        
+        if width is not None:
+            self.__cache_width(publisher, width)
+            return width
+        
+        width = self.__read_width(publisher)
+            
+        return width
+    
+    
+    def __read_width(self, publisher:str):
+        """
+        reads the width from the config file for the supplied publisher. \
+        If not found, then use the ACL format width
+
+        Args:
+            publisher (str): the publisher name
+
+        Returns:
+            float: the width in pts
+        """
+        if publisher in self.config['width']:
+            width = self.config['width'][publisher]
         else:
-            print(f'[INFO] Publisher "{publisher}" not found, setting to ACL format!')
+            print(f'[INFO] Publisher "{publisher}" not found, setting to ACL format! \
+                  To save this publisher, give the values for both publisher and width')
             width = 455.244
             
         return width
+    
+    
+    def __cache_width(self, publisher:str, width:float):
+        """
+        caches the publisher:width mapping 
+
+        Args:
+            publisher (str): the publisher
+            width (float): the width in pts
+        """
+        self.config['width'][publisher] = width
+        save_file(self.config, Sizing.CONFIG_PATH)
 
 
     def adjust_font_size(self, subplots, fraction, **kwargs):
-        """
-        adjust the font_size based on the fraction and number of columns and \
+        """adjust the font_size based on the fraction and number of columns and
         saves the adjusted params
 
-        :param subplots: at uple of (nrow, ncols)
-        :param fraction: fraction of the total width to use
+        Args:
+            subplots: at uple of (nrow, ncols)
+            fraction: fraction of the total width to use
         """
         _, num_cols = subplots
         
@@ -70,15 +122,18 @@ class Sizing():
         
         
     def __find_matching_param(self, key, main_params, special_params={}):
-        """
-        find the matching paramater to the key in the matplotlib.rcParams file specified by \
+        """find the matching paramater to the key in the matplotlib.rcParams file specified by
         the main_params and special_params
 
-        :param key: the key with which we search
-        :param main_params: the main dictionary, with keys as rcParams keys and values as their shortened versions
-        :param special_params: direct mapping to handle exceptions where probabilstic matching might fail, defaults to {}
-        
-        :return: the found parameter, or None if not found
+        Args:
+            key: the key with which we search
+            main_params: the main dictionary, with keys as rcParams keys
+                and values as their shortened versions
+            special_params: direct mapping to handle exceptions where
+                probabilstic matching might fail, defaults to {}
+
+        Returns:
+            the found parameter, or None if not found
         """
         all_matches = list(main_params.keys()) + list(main_params.values()) # Search over both keys and values
         
@@ -100,14 +155,15 @@ class Sizing():
         
 
     def update_textsize(self, reinitialize=True, **kwargs):
-        """
-        This function changes the font size of various text elements in the plot such as xlabel, \
-        ylabel, title, sticks, etc., by a specified offset. Uses deterministic and probabilistic \
+        """This function changes the font size of various text elements in the plot such as xlabel,
+        ylabel, title, sticks, etc., by a specified offset. Uses deterministic and probabilistic
         matching to determine and map the keyword argument to the required property
-        
-        :param reinitialize: initialize size params to those created by `get_size` after adjusting for\
-            columns, fraction etc.
-        :param kwargs: flexible keyword arguments specified by the user, where the name of \
+
+        Args:
+            reinitialize: initialize size params to those created by
+                `get_size` after adjusting for columns, fraction etc.
+            **kwargs: flexible keyword arguments specified by the user,
+                where the name of
         the argument is the key and the offset is the value.
         """
         if reinitialize:
@@ -133,15 +189,16 @@ class Sizing():
             
             
     def update_textweight(self, reinitialize=False, **kwargs):
-        """
-        This function changes the font weight of various text elements in the plot such as xlabel, \
+        """This function changes the font weight of various text elements in the plot such as xlabel,
         ylabel, title. The values are 'light', 'normal', 'bold'.
-        Uses deterministic and probabilistic matching to determine and map the keyword 
-        \ argument to the required property
-        
-        :param reinitialize: initialize size params to those created by `get_size` after adjusting for\
-            columns, fraction etc.
-        :param kwargs: flexible keyword arguments specified by the user, where the name of \
+        Uses deterministic and probabilistic matching to determine and map the keyword
+        argument to the required property
+
+        Args:
+            reinitialize: initialize size params to those created by
+                `get_size` after adjusting for columns, fraction etc.
+            **kwargs: flexible keyword arguments specified by the user,
+                where the name of
         the argument is the key and the weight is the value.
         """
         if reinitialize:
@@ -162,11 +219,11 @@ class Sizing():
     
 
     def remove_ticks(self, xtick=True, ytick=True):
-        """
-        remove the tick marks 
+        """remove the tick marks
 
-        :param xtick: remove ticks on x-axis, defaults to True
-        :param ytick: remove ticks on y-axis, defaults to True
+        Args:
+            xtick: remove ticks on x-axis, defaults to True
+            ytick: remove ticks on y-axis, defaults to True
         """
         if xtick:
             plt.rcParams['xtick.major.size'] = 0
@@ -175,14 +232,14 @@ class Sizing():
             
             
     def set_lim(self, ax, xlims=(None, None), ylims=(None, None)):
-        """
-        set the starting and ending points of the x and y axes
+        """set the starting and ending points of the x and y axes
 
-        :param ax: the matplotlib axis object
-        :param xlims: tuple containing the (min, max) values of the \
-                        x axis, defaults to (None, None)
-        :param ylims: tuple containing the (min, max) values of the \
-                        y axis, defaults to (None, None)
+        Args:
+            ax: the matplotlib axis object
+            xlims: tuple containing the (min, max) values of the x
+                axis, defaults to (None, None)
+            ylims: tuple containing the (min, max) values of the y
+                axis, defaults to (None, None)
         """
         if xlims[0] is not None and xlims[1] is not None:
             ax.set_xlim(xlims[0], xlims[1])
@@ -191,14 +248,16 @@ class Sizing():
 
         
     def convert_width_to_inches(self, width=None, publisher=None):
-        """
-        convert width from pts to inches
+        """convert width from pts to inches
 
-        :param width: the given width in pts, defaults to None
-        :param publisher: the name of the publisher, defaults to None
-        :return: the width in inches
+        Args:
+            width: the given width in pts, defaults to None
+            publisher: the name of the publisher, defaults to None
+
+        Returns:
+            the width in inches
         """
-        width_pt = width or self.__get_width_publisher(publisher=publisher)
+        width_pt = width or self.__get_width_publisher(publisher=publisher, width=width)
         fig_width_pt = width_pt 
         inches_per_pt = 1 / 72.27
 
@@ -208,17 +267,22 @@ class Sizing():
         
     def get_size(self, width=None, publisher=None, width_in_pts=True, reinitialize=True, fraction=1, 
                    subplots=(1, 1), **kwargs):
-        """
-        finds the ideal size of the plot and adjusts the text sizes according to the required dimensions
+        """finds the ideal size of the plot and adjusts the text sizes according to the required dimensions
+            if both publisher and width are specified, then the publisher:width is cached
+            
+        Args:
+            width: the width, defaults to None
+            publisher: the name of the publisher, defaults to None
+            width_in_pts: whether the width is in points, defaults to
+                True
+            reinitialize: whether to reinitialize the config to itys
+                defaults, defaults to True
+            fraction: the fraction of the width supplied to use,
+                defaults to 1
+            subplots: (nrows, ncols), defaults to (1, 1)
 
-        :param width: the width, defaults to None
-        :param publisher: the name of the publisher, defaults to None
-        :param width_in_pts: whether the width is in points, defaults to True
-        :param reinitialize: whether to reinitialize the config to itys defaults, defaults to True
-        :param fraction: the fraction of the width supplied to use, defaults to 1
-        :param subplots: (nrows, ncols), defaults to (1, 1)
-        
-        :return: figsize (width, height)
+        Returns:
+            figsize (width, height)
         """
         assert publisher is not None or width is not None, "Either set the width or the format."
 
